#!/bin/sh
#20240314

. /etc/DISTRO_SPECS
TODAYDATE="`date +%Y%m%d`"
echo -n "$TODAYDATE" > TODAYDATE

echo -n "Please type in the kernel version [5.4.1]: "
read KERNVER
DOTcnt=$(echo -n "${KERNVER//[^.]/}" | wc -m)
if [ $DOTcnt -eq 1 ];then
 KVER12="${KERNVER}"
else
 KVER12="${KERNVER%.*}" #ex: 6.8
fi

CURDIR="`pwd`"
if [ "$CURDIR" != "/usr/src" ];then
 echo 'This script must be in /usr/src'
 exit
fi
if [ ! -f linux-${KVER12}.tar.xz ];then
 echo "Please download linux-${KVER12}.tar.xz"
 exit
fi
if [ ! -f DOTconfig ];then
 echo 'Please download DOTconfig, the kernel configuration file'
 echo 'If you have another file that you want to use, rename it to DOTconfig'
 exit
fi

rm -rf temp1 2>/dev/null
mkdir temp1

rm -rf out-k* #get rid of earlier builds.

#patch kernel
echo
echo 'Extracting and patching kernel...'
rm -rf linux-${KVER12} 2> /dev/null
rm -rf linux-${KVER12} 2> /dev/null
sync
tar -xf linux-${KVER12}.tar.xz
sync

if [ -f patches/patch-${KERNVER}.xz ];then
 echo
 cp -f patches/patch-${KERNVER}.xz ./
 unxz patch-${KERNVER}.xz
 sync

 cd linux-${KVER12}
 patch -p1 < ../patch-${KERNVER}
 cd ..
 sync
 rm patch-${KERNVER}
fi

ln -snf linux-${KVER12} linux-${KERNVER}
ln -snf linux-${KVER12} linux

#TEST
echo -n 'Press ENTER: '
read keepgoing

#20240115 aufs6
if [ "`ls 3rd-party/filesystem/aufs6*.tar.gz 2>/dev/null`" != "" ];then
 echo 'aufs patch...'
 cp -f 3rd-party/filesystem/aufs6*.tar.gz temp1/aufs.tar.gz
 cd temp1
 tar -zxf aufs.tar.gz
 DIRNAME="`find -mindepth 1 -maxdepth 1 -type d -name 'aufs6*'`"
 cd $DIRNAME
 cp -f aufs6-base.patch ../../
 cp -f aufs6-kbuild.patch ../../
 #if vercmp ${KERNELVER3} lt 5.4;then cp -f aufs6-standalone.patch ../../ ; fi
 #cp -f aufs6-standalone.patch ../../
 #cp -f aufs6-loopback.patch ../../
 cp -f aufs6-mmap.patch ../../ #131204
 cd ../../linux-${KERNVER}
 patch -p1 < ../aufs6-kbuild.patch
 patch -p1 < ../aufs6-base.patch
 #if vercmp ${KERNELVER3} lt 5.4;then patch -p1 < ../aufs6-standalone.patch ; fi
 #patch -p1 < ../aufs6-standalone.patch
 #patch -p1 < ../aufs6-loopback.patch
 patch -p1 < ../aufs6-mmap.patch #131204
 cp -a -f ../temp1/$DIRNAME/Documentation ./
 cp -a -f ../temp1/$DIRNAME/fs ./
 cp -a -f ../temp1/$DIRNAME/include/linux/aufs_type.h ./include/linux/ 2>/dev/null #131204 fails.
 [ -d ../temp1/$DIRNAME/include/uapi/linux ] && cp -a -f ../temp1/$DIRNAME/include/uapi/linux/aufs_type.h ./include/uapi/linux/ #130220
 cd ..
 gzip aufs6-kbuild.patch
 gzip aufs6-base.patch
 #if vercmp ${KERNELVER3} lt 5.4;then gzip aufs6-standalone.patch ; fi
 #gzip aufs6-standalone.patch
 gzip aufs6-mmap.patch
 mv -f aufs6-kbuild.patch.gz patches/
 mv -f aufs6-base.patch.gz patches/
 #if vercmp ${KERNELVER3} lt 5.4;then mv -f aufs6-standalone.patch.gz patches/ ; fi
 #mv -f aufs6-standalone.patch.gz patches/
 mv -f aufs6-mmap.patch.gz patches/
 #mv -f aufs6-loopback.patch.gz patches/
 
 #aufs-allow-sfs-in-branch
 echo
 echo 'aufs-allow-sfs-in-branch patch...'
 cp patches/aufs-allow-sfs.patch.gz ./
 gunzip aufs-allow-sfs.patch.gz
 cd linux-${KERNVER}
 patch -p1 < ../aufs-allow-sfs.patch
 cd ..
 rm aufs-allow-sfs.patch
fi

if [ -f 3rd-party/filesystem/btrfs-fscrypt.tar.gz ];then
 echo "btrfs fscrypt patch..."
 cp -a -f 3rd-party/filesystem/btrfs-fscrypt.tar.gz temp1/
 cd temp1
 tar -xf btrfs-fscrypt.tar.gz
 cd ..
 cd linux-${KERNVER}
 Psorted="$(find ../temp1/btrfs-fscrypt -type f -name '*.patch' | sort -n | tr '\n' ' ')"
 for aP in ${Psorted}
 do
  patch -p1 < ${aP}
 done
 cd ..
 rm -f temp1/btrfs-fscrypt.tar.gz
 rm -rf temp1/btrfs-fscrypt
fi

if [ -f patches/cap_sys_mount-1.patch.gz ];then
 echo
 echo "cap_sys_mount patch..."
 cp -f patches/cap_sys_mount-1.patch.gz ./
 gunzip cap_sys_mount-1.patch.gz
 cp -f patches/cap_sys_mount-2.patch.gz ./
 gunzip cap_sys_mount-2.patch.gz
 cd linux-${KERNVER}
 patch -p1 < ../cap_sys_mount-1.patch
 patch -p1 < ../cap_sys_mount-2.patch
 cd ..
 rm cap_sys_mount-1.patch
 rm cap_sys_mount-2.patch
fi

#create tarball of patched source
sync
echo
echo 'Creating kernel patched source tarball...'
rm -f linux-${KERNVER}-src-patched.tar.bz2 2>/dev/null
tar -c -f linux-${KERNVER}-src-patched.tar linux-${KERNVER}/
bzip2 linux-${KERNVER}-src-patched.tar
sync

#compile kernel
echo
echo -n 'ENTER to configure kernel: '
read goforit
cd linux-${KERNVER}
make clean
make mrproper
cp -f ../DOTconfig .config
make menuconfig
sync
sleep 1
cp -f .config DOTconfig-${KERNVER}
cp -f DOTconfig-${KERNVER} ../
echo -n 'ENTER to compile kernel: '
read goforit
make
sync

#place to send everything...
rm -rf /usr/src/out-k${KERNVER} 2> /dev/null
mkdir /usr/src/out-k${KERNVER}

#install kernel
echo
echo -n "ENTER to install kernel: "
read goforit
echo
echo "Installing modules..."

rm -rf /lib/modules/${KERNVER} 2> /dev/null
rm -rf /lib/modules/${KVER12} 2> /dev/null
#rm -rf /lib/firmware-OLD 2> /dev/null
#mv -f /lib/firmware /lib/firmware-OLD
#mkdir /lib/firmware
make INSTALL_MOD_STRIP=1 modules_install #strip the modules.
sync
echo "Creating headers tarball in /usr/src/out-k${KERNVER}..."
make headers_install
sync
mkdir -p /usr/src/out-k${KERNVER}/linux_headers-${KERNVER}-${TODAYDATE}/usr/include
cp -a -f usr/include/* ../out-k${KERNVER}/linux_headers-${KERNVER}-${TODAYDATE}/usr/include/
sync

#for 6.8 kernel, installed /lib/modules/6.8.0
if [ ! -d /lib/modules/${KERNVER} ];then
 MODVER="${KERNVER}.0"
 ln -s out-k${KERNVER} ../out-k${MODVER}
 ln -s linux-${KERNVER} ../linux-${MODVER}
else
 MODVER="${KERNVER}"
fi

#turn headers into a pet...
HDRSIZE=$(du -s -k ../out-k${KERNVER}/linux_headers-${KERNVER}-${TODAYDATE} | cut -f 1)
echo "linux_headers-${KERNVER}-${TODAYDATE}|linux_headers|${KERNVER}-${TODAYDATE}||BuildingBlock|${HDRSIZE}K||linux_headers-${KERNVER}-${TODAYDATE}.pet||header files for ${KERNVER} kernel||||" > ../out-k${KERNVER}/linux_headers-${KERNVER}-${TODAYDATE}/pet.specs

echo "Creating tarball of modules..."
cp -a -f /lib/modules/${MODVER} ../out-k${KERNVER}/

echo "Copying DOTconfig-${KERNVER}, System.map, vmlinuz..."
cp -f DOTconfig-${KERNVER} ../out-k${KERNVER}/
cp -f System.map ../out-k${KERNVER}/
cp -f modules.order ../out-k${KERNVER}/
cp -f modules.builtin ../out-k${KERNVER}/ 2>/dev/null #<=2.6.32 does not have this.
cp -f arch/x86/boot/bzImage ../out-k${KERNVER}/vmlinuz
sync
cd ../out-k${KERNVER}
dir2tgz linux_headers-${KERNVER}-${TODAYDATE}
sync
tgz2pet linux_headers-${KERNVER}-${TODAYDATE}.tar.gz
dir2tgz ${KERNVER}
sync
rm -rf linux_headers-${KERNVER}-${TODAYDATE}
rm -rf ${KERNVER}
cd ..
cp -f -a patches out-k${KERNVER}/
cp -f -a 3rd-party out-k${KERNVER}/
mv -f DOTconfig DOTconfig-PREVIOUS
cp -f -a DOTconfig-${KERNVER} DOTconfig
sync

mv -f linux-${KERNVER}-src-patched.tar.bz2 out-k${KERNVER}/
sync

echo "KERNVER='${KERNVER}'
MODVER='${MODVER}'" > VERS

echo
echo "Finished!"
echo "The next step is to run '2modules.sh' to compile 3rd-party modules."
echo "However, EasyOS must be running with the new kernel and this will"
echo "require a reboot. Just press ENTER to copy 'vmlinuz' to /boot"
echo "This script will then exit, and you will have to copy vmlinuz to"
echo "the boot partition, then reboot."
echo -n "Press ENTER: "
read YESGO
if [ "$YESGO" = "" ];then
 OLDKER="`uname -r`"
 mkdir -p /boot
 [ -f /boot/vmlinuz ] && mv -f /boot/vmlinuz /boot/vmlinuz-${OLDKER}-OLD
 cp -f out-k${KERNVER}/vmlinuz /boot/
 echo "Copied 'vmlinuz' to /boot, copy it to wherever boots from, then you must reboot."
fi

echo "1kernel.sh finished"
###end###
