#!/bin/sh
#run this script after running 1kernel.sh, then after a reboot.
#20240314

. /etc/DISTRO_SPECS

KERNELVERSION="`uname -r`"
[ ! -d linux-${KERNELVERSION} ] && exit
TODAYDATE="$(cat TODAYDATE)"

echo "System currently running with ${KERNELVERSION} kernel."
echo "The modules will be compiled for this kernel."
echo

MACHINENAME="`uname -m`" #ex: x86_64

#3rd-party drivers

#110827 nvidia
if [ -d 3rd-party/nvidia ];then #120125
 cd 3rd-party/nvidia
 export SYSSRC=/usr/src/linux-${KERNELVERSION}
 export KERNEL_MODLIB=/lib/modules/${KERNELVERSION}
 export KERNEL_UNAME=${KERNELVERSION}
 export MODULE_ROOT=/lib/modules/${KERNELVERSION}
 export SYSOUT=/usr/src/linux-${KERNELVERSION}
 export CC=cc
 for ONENV in `ls -1 | tr '\n' ' '` #ex: NVIDIA-Linux-x86-195.36.31-pkg1.run NVIDIA-Linux-x86-260.19.12.run NVIDIA-Linux-x86-71.86.14-pkg1.run
 do
  sh ./${ONENV} --extract-only
  if [ $? -eq 0 ];then
   NVBASE="`basename $ONENV .run`"
   NVVER="`echo -n "$ONENV" | cut -f 4 -d '-'`" #ex: 195.36.31
   NVVER="`basename $NVVER .run`" #one case, still has .run on end.
   if [ -d ${NVBASE}/usr/src/nv ];then
    cd ${NVBASE}/usr/src/nv
    make module
    sync
    mkdir -p ../../../../../../out-k${KERNELVERSION}/nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}/lib/modules/${KERNELVERSION}/nvidia
    cp nvidia.ko ../../../../../../out-k${KERNELVERSION}/nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}/lib/modules/${KERNELVERSION}/nvidia/
    SIZEK="`du -k -s ../../../../../../out-k${KERNELVERSION}/nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}/lib/modules/${KERNELVERSION} | cut -f 1`"
    #db fields: pkgname|nameonly|version|pkgrelease|category|size|path|fullfilename|dependencies|description|compileddistro|compiledrelease|repo|
    echo "nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}|nvidia_kmodule|${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}||Setup|${SIZEK}K||nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}.pet|+linux_kernel&eq${KERNELVERSION},+nvidia_base&eq${NVVER}|commercial kernel nvidia driver, needs nvidia_base pkg, runs on kernel ${KERNELVERSION}, compiled in ${DISTRO_NAME} version ${DISTRO_VERSION}|${DISTRO_BINARY_COMPAT}|${DISTRO_COMPAT_VERSION}||" > ../../../../../../out-k${KERNELVERSION}/nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}/pet.specs
    cd ../../../../
   else
    cd ${NVBASE}/kernel
    make module
    sync
    mkdir -p ../../../../out-k${KERNELVERSION}/nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}/lib/modules/${KERNELVERSION}/nvidia
    cp nvidia.ko ../../../../out-k${KERNELVERSION}/nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}/lib/modules/${KERNELVERSION}/nvidia/
    SIZEK="`du -k -s ../../../../out-k${KERNELVERSION}/nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}/lib/modules/${KERNELVERSION} | cut -f 1`"
    #db fields: pkgname|nameonly|version|pkgrelease|category|size|path|fullfilename|dependencies|description|compileddistro|compiledrelease|repo|
    echo "nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}|nvidia_kmodule|${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}||Setup|${SIZEK}K||nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}.pet|+linux_kernel&eq${KERNELVERSION},+nvidia_base&eq${NVVER}|commercial kernel nvidia driver, needs nvidia_base pkg, runs on kernel ${KERNELVERSION}, compiled in ${DISTRO_NAME} version ${DISTRO_VERSION}|${DISTRO_BINARY_COMPAT}|${DISTRO_COMPAT_VERSION}||" > ../../../../out-k${KERNELVERSION}/nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}/pet.specs
    cd ../../
   fi
   sync
   rm -rf ./${NVBASE}
  fi
 done
 cd ../../
fi

#20220426 
DROIDCAM="$(find 3rd-party/misc -maxdepth 1 -type f -name 'droidcam*tar.gz')"
if [ "$DROIDCAM" ];then
 echo 'Compiling droidcam v4l2loopback-dc driver...'
 NAMEONLY="$(basename $DROIDCAM .tar.gz)"
 cd 3rd-party/misc
 tar -xf ${NAMEONLY}.tar.gz
 cd ${NAMEONLY}
 make
 if [ -f v4l2loopback-dc.ko ];then
  make install
  #...no, that installs to /lib/modules/${KERNELVERSION}/extra
  #mkdir -p /lib/modules/${KERNELVERSION}/kernel/drivers/media/video
  #cp -a -f v4l2loopback-dc.ko /lib/modules/${KERNELVERSION}/kernel/drivers/media/video/
  #...have left it at 'make install' as think that signs the module.
  # yes, ref: https://www.kernel.org/doc/html/v4.15/admin-guide/module-signing.html
 fi
 sync
 cd ..
 rm -rf ./${NAMEONLY}
 cd ../../
fi

#20201202  20240205 ref: https://github.com/antoineco/broadcom-wl
STA_WL="$(find 3rd-party/wireless -maxdepth 1 -type f -name 'broadcom-wl-*z')"
if [ "$STA_WL" ];then
 echo "Compiling Broadcom STA WL driver (wl.ko)..."
 PKGNAME="$(basename $STA_WL)"
 cd 3rd-party/wireless
 tar -xf $PKGNAME
 if [ -d broadcom-wl-6.6 ];then
  cd broadcom-wl-6.6
  make
  sync
  if [ -f wl.ko ];then
   #create a separate PET for this...
   mkdir -p broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64/lib/modules/${KERNELVERSION}/kernel/drivers/net/wireless
   cp -a -f wl.ko broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64/lib/modules/${KERNELVERSION}/kernel/drivers/net/wireless/
   echo '!#/bin/sh
if [ "`pwd`" == "/" ];then
 depmod
fi' > broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64/pinstall.sh
   chmod 755 broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64/pinstall.sh
   echo '!#/bin/sh
depmod' > broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64/puninstall.sh
   chmod 755 broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64/puninstall.sh
   echo "broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64|broadcom-sta-wl|20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64||BuildingBlock|7380K||broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64.pet|+linux_kernel&eq${KERNELVERSION}|Broadcom wl.ko alternative driver|${DISTRO_BINARY_COMPAT}|${DISTRO_COMPAT_VERSION}||" > broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64/pet.specs
   dir2tgz broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64
   tgz2pet broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64.tar.gz
   mv -f broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64.pet ../../../out-k${KERNELVERSION}/
   mv -f broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64 ../../../out-k${KERNELVERSION}/
   echo "PET created: out-k${KERNELVERSION}/broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64.pet"
   echo "...wl.ko will not be included in main PET, as conflicts with other drivers."
  else
   echo "...Sorry, compile failed, wl.ko not created"
  fi
  sync
  cd ..
  rm -rf broadcom-wl-6.6
 else
  echo "...Sorry, directory 'broadcom-wl-6.6' does not exist, cannot compile."
 fi
 cd ../../
fi

##20220403 rtw89 has installed some firmware, code delayed from 1kernel.sh...
#echo "Creating 'firmware' tarball in /usr/src/out-k${KERNELVERSION}..."
#cp -a -f /lib/firmware out-k${KERNELVERSION}/
#cd out-k${KERNELVERSION}
#dir2tgz firmware/
#sync
#rm -rf firmware
#cd ..

depmod
sync
echo "Current directory: `pwd`"
echo -n "Press ENTER to continue: "
read keepgoing

echo "Creating tarball of modules, out-k${KERNELVERSION}/${KERNELVERSION}.tar.gz ..."
cp -a -f /lib/modules/${KERNELVERSION} out-k${KERNELVERSION}/
cd out-k${KERNELVERSION}
[ -f ${KERNELVERSION}.tar.gz ] && mv -f ${KERNELVERSION}.tar.gz ${KERNELVERSION}.tar.gzWITHOUT_3RDPARTY
dir2tgz ${KERNELVERSION}
sync
rm -rf ${KERNELVERSION}
cd ..

#110827
echo
echo "make commercial nvidia modules into pets..."
cd out-k${KERNELVERSION}
for ONENV in `find . -maxdepth 1 -type d -name 'nvidia_kmodule*' | sed -e 's%^\./%%' | tr '\n' ' '`
do
 dir2tgz $ONENV
 tgz2pet ${ONENV}.tar.gz
 sync
done
cd ..

echo
echo "Finished. Now run 3package.sh"

###end###
